/**************************************************************************************************************************
// This script was authored by Lorne Brooks - Email: lbtools2k@gmail.com
//
//
// REVISION HISTORY:
// Version 1.2 27/10/14;
//						- Email change for help contact.
// Version 1.1 23/10/14;
//						- now operates on a single noise controller assigned either directly to the root position or rotation tracks or as part
//							of a list controller on any of the subanim tracks. Ignores multiple noise controllers and only uses the first one
//							found. Does not handle nested list controllers or noise float tracks on XYZ controllers.
// Version 1.0 10/10/14;
//						- First public release.
//
//
// This script is not copyrighted, please feel free to modify or distribute.
//
**************************************************************************************************************************/

global g_LBTools_NM

struct s_LBTools_noiseManager
(
/*==================================================================================
// Structure variables
====================================================================================*/

noiseManager_Rollout,
noiseManagerSS_Rollout,
aboutScript_rollout,
a_SS_Items = #(), --// This array gets modified by the selection set interface and operated on by the main interface
dPos,

/*==================================================================================
// Structure functions
/*==================================================================================

f_refreshSS_UI
f_findNoiseTrack_Pos
f_findNoiseTrack_Rot
f_noiseManager
	
====================================================================================*/

fn f_refreshSS_UI =
(
	if selectionSets.count > 0 then
	(
		try --// Try/Catch in case of invalid scene data
		(
			--// Keep track of previously selected set or node items
			local selSetIndex = g_LBTools_NM.noiseManagerSS_Rollout.ddl_selSets.selection
			local ba_selItems = g_LBTools_NM.noiseManagerSS_Rollout.mlb_objNames.selection
			
			--// Reopen the selection sets interface
			try(destroyDialog g_LBTools_NM.noiseManagerSS_Rollout)catch()
			createDialog g_LBTools_NM.noiseManagerSS_Rollout width:360 pos:((GetDialogPos g_LBTools_NM.noiseManager_Rollout) + [175,0])
			g_LBTools_NM.noiseManager_Rollout.chkbtn_selectionSets.checked = true
			
			--// Reselect any preivous selection set and node items
			g_LBTools_NM.noiseManagerSS_Rollout.ddl_selSets.selection = selSetIndex
			
			--// Update the node items list and reselect the nodes
			local a_setItems = #()
			local itemCount = getNamedSelSetItemCount selSetIndex
			if itemCount > 0 do
			(
				for i = 1 to itemCount do
				(
					append a_setItems (getNamedSelSetItem selSetIndex i).name
				)
			)
			g_LBTools_NM.noiseManagerSS_Rollout.mlb_objNames.items = a_setItems
			g_LBTools_NM.noiseManagerSS_Rollout.mlb_objNames.selection = ba_selItems
			g_LBTools_NM.noiseManagerSS_Rollout.f_populateFlags()
			g_LBTools_NM.noiseManagerSS_Rollout.f_populateItemsArray()
		)
		catch
		(
			g_LBTools_NM.noiseManagerSS_Rollout.ddl_selSets.selection = 1
			messageBox "Error: Scene data has changed while the interface has been open.\n\nVerify the following cases:\n- Selection sets still exist and contain objects\n- Objects in sets have not been deleted\n- The scene has not been reset\n- New selection sets with objects have been added\n- A scene was loaded containing selection sets with objects" title:""
		)
	)
	else
	(
		messageBox "Error: No selection sets found." title:""
		g_LBTools_NM.noiseManagerSS_Rollout.ddl_selSets.items = #()
		g_LBTools_NM.noiseManagerSS_Rollout.mlb_objNames.items = #()
		g_LBTools_NM.noiseManagerSS_Rollout.f_populateFlags()
	)
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_findNoiseTrack_Pos obj =
(
	--// Returns the index of the first found noise track on the subanim controller of a list controller. Returns 0 for noise found on the root position track, otherwise undefined
	if classof obj.position.controller == Noise_Position then --// Found noise on position root track
	(
		0
	)	
	else --// Check for noise in a list controller
	(
		local nt_index
		for i = 1 to obj.position.controller.numsubs do
		(
			if classof obj.position.controller[i].controller == Noise_Position do
			(
				nt_index = i
				exit
			)
		)
		if nt_index != undefined then --// Noise position controller found within a list controller
		(
			nt_index
		)
		else --// No noise position controller found
		(
			undefined
		)
	)
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_findNoiseTrack_Rot obj =
(
	--// Returns the index of the first found noise track on the subanim controller of a list controller. Returns 0 for noise found on the root rotation track, otherwise undefined
	if classof obj.rotation.controller == Noise_Rotation then --// Found noise on rotation root track
	(
		0
	)	
	else --// Check for noise in a list controller
	(
		local nt_index
		for i = 1 to obj.rotation.controller.numsubs do
		(
			if classof obj.rotation.controller[i].controller == Noise_Rotation do
			(
				nt_index = i
				exit
			)
		)
		if nt_index != undefined then --// Noise rotation controller found within a list controller
		(
			nt_index
		)
		else --// No noise rotation controller found
		(
			undefined
		)
	)
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_noiseManager =
(
	rollout noiseManagerSS_Rollout "Noise Manager - Selection Sets"
	(
		--// NOTE: Currently the display lists for the selection set items and list box node items are not name sorted - it uses the internal order-of-creation instead
		
		label lbl_ss "Selection sets:" align:#left
		dropdownlist ddl_selSets "" width:310 across:2
		button btn_refresh "R" width:18 height:18 align:#right offset:[0,2] tooltip:"Refresh the set and object lists in case the scene data has changed."
		label lbl_obj "Select objects in the set list to operate on:" align:#left
		button btn_selectAll "Select All" width:140 height:16 align:#left across:4
		button btn_selectNone "Select None" width:140 height:16 align:#left offset:[60,0]
		label lbl_pos "Pos" align:#right offset:[57,0]
		label lbl_rot "Rot" align:#right offset:[0,0]
		multiListBox mlb_objNames "" width:285 height:36 align:#left across:3
		multiListBox mlb_posCtrl "" width:26 height:36 align:#right offset:[88,0]
		multiListBox mlb_rotCtrl "" width:26 height:36 align:#right offset:[2,0]
		
		on noiseManagerSS_Rollout moved pos do
		(
			SetDialogPos g_LBTools_NM.noiseManager_Rollout (pos + [-175,0])
		)
		
		fn f_populateFlags =
		(
			--// Populate the postion and rotation display flags
			local a_posFlags = #()
			for str_itemName in mlb_objNames.items do
			(
				try --// Try/Catch in case of invalid nodes (like Biped parts)
				(
					local obj = getNodeByName str_itemName
					local nt_index = g_LBTools_NM.f_findNoiseTrack_Pos obj
					if nt_index != undefined then
					(
						append a_posFlags "YES"
					)
					else
					(
						append a_posFlags "  -"
					)
				)catch()
			)
			local a_rotFlags = #()
			for str_itemName in mlb_objNames.items do
			(
				try --// Try/Catch in case of invalid nodes (like Biped parts)
				(
					local obj = getNodeByName str_itemName
					local nt_index = g_LBTools_NM.f_findNoiseTrack_Rot obj
					if nt_index != undefined then
					(
						append a_rotFlags "YES"
					)
					else
					(
						append a_rotFlags "  -"
					)
				)catch()
			)
			mlb_posCtrl.items = a_posFlags
			mlb_rotCtrl.items = a_rotFlags
		)
		
		on noiseManagerSS_Rollout open do
		(
			if selectionSets.count > 0 then
			(
				clearSelection()
				timeConfiguration.playbackLoop = true --// This is the default interface option
				local a_selSets = for i = 1 to selectionSets.count collect (getNamedSelSetName i)
				ddl_selSets.items = a_selSets
				local a_setItems = #()
				local itemCount = getNamedSelSetItemCount 1
				if itemCount > 0 do
				(
					for i = 1 to itemCount do
					(
						append a_setItems (getNamedSelSetItem 1 i).name
					)
				)
				mlb_objNames.items = a_setItems
				mlb_objNames.selection = #{}
				f_populateFlags()
			)
			else
			(
				messageBox "Error: No selection sets found." title:""
			)
		)
		
		on noiseManagerSS_Rollout close do
		(
			g_LBTools_NM.a_SS_Items = #() --// Reset the array when the UI is closed to return to viewport selected mode
			g_LBTools_NM.noiseManager_Rollout.chkbtn_selectionSets.checked = false
		)
		
		on ddl_selSets selected setIndex do
		(
			try
			(
				mlb_posCtrl.selection = #{}
				mlb_rotCtrl.selection = #{}
				g_LBTools_NM.a_SS_Items = #() --// Reset the array each time a new selection is made
				clearSelection()
				--// In case the selection set scene data has changed - repopulate the sets list
				local a_selSets = for i = 1 to selectionSets.count collect (getNamedSelSetName i)
				ddl_selSets.items = a_selSets
				--// Collect and display the node items found in the selected set
				local a_setItems = #()
				local itemCount = getNamedSelSetItemCount setIndex
				if itemCount > 0 do
				(
					for i = 1 to itemCount do
					(
						append a_setItems (getNamedSelSetItem setIndex i).name
					)
				)
				mlb_objNames.items = a_setItems
				mlb_objNames.selection = #{}
				f_populateFlags()
			)
			catch --// In case the scene data no longer exists, refresh the interface
			(
				g_LBTools_NM.f_refreshSS_UI()
			)
		)
		
		on btn_refresh pressed do
		(
			g_LBTools_NM.f_refreshSS_UI()
		)
		
		fn f_populateItemsArray =
		(
			try
			(
				g_LBTools_NM.a_SS_Items = #() --// Reset the array each time a new selection is made
				--// Convert the bitarray to nodes and collect them for noise operations in the main rollout
				for i in mlb_objNames.selection do
				(
					append g_LBTools_NM.a_SS_Items (getNamedSelSetItem ddl_selSets.selection i)
				)
				try(select g_LBTools_NM.a_SS_Items)catch(messageBox "Error: Can't select objects.\n" title:"")
			)
			catch --// In case the scene data no longer exists, refresh the interface
			(
				g_LBTools_NM.f_refreshSS_UI()
			)
		)
		
		on mlb_objNames selectionEnd do
		(
			mlb_posCtrl.selection = #{}
			mlb_rotCtrl.selection = #{}
			f_populateItemsArray()
		)
		
		on btn_selectAll pressed do
		(
			g_LBTools_NM.f_refreshSS_UI() --// Refresh the interface in case scene data has changed
			mlb_objNames.selection = #{1..mlb_objNames.items.count}
			f_populateItemsArray()
		)
		
		on btn_selectNone pressed do
		(
			g_LBTools_NM.f_refreshSS_UI() --// Refresh the interface in case scene data has changed
			mlb_objNames.selection = #{}
			g_LBTools_NM.a_SS_Items = #() --// Reset the array
			clearSelection()
		)
	) --// end rollout
	
	rollout noiseManager_Rollout "Noise Manager"
	(
		group "Tracks to affect:"
		(
			checkbox chkbx_pos "Position" checked:false across:2 align:#left offset:[0,0]
			checkbox chkbx_rot "Rotation" checked:true align:#left offset:[10,0]
		)
		group "Noise settings:"
		(
			spinner spn_seed "Seed: " range:[0,100000000,0] type:#integer align:#right
			spinner spn_frequency "Frequency: " scale:0.001 range:[0.001,10.0,0.5] align:#right
			spinner spn_rampIn "Ramp in: " range:[0,100000000,0] type:#integer align:#right
			spinner spn_rampOut "Ramp out: " range:[0,100000000,0] type:#integer align:#right
			label lbl_txt2 "Strength:" align:#left
			spinner spn_strX "X: " scale:0.001 range:[-1000000.0,1000000.0,45.0] align:#left offset:[5,0]
			spinner spn_strY "Y: " scale:0.001 range:[-1000000.0,1000000.0,45.0] align:#left offset:[5,0]
			spinner spn_strZ "Z: " scale:0.001 range:[-1000000.0,1000000.0,45.0] align:#left offset:[5,0]
			checkbox chkbx_posX "" align:#right offset:[53,-63] tooltip:"Limits the X strength value to above 0." across:2
			label lbl_posX " >0" align:#right offset:[-2,-63]
			checkbox chkbx_posY "" align:#right offset:[53,1] tooltip:"Limits the Y strength value to above 0." across:2
			label lbl_posY " >0" align:#right offset:[-2,1]
			checkbox chkbx_posZ "" align:#right offset:[53,2] tooltip:"Limits the Z strength value to above 0." across:2
			label lbl_posZ " >0" align:#right offset:[-2,2]
			checkbutton chkbtn_lock "> Lock Strength Values <" checked:false width:140 height:22 align:#center tooltip:"Activate to change all of the strength spinners to the same value."
			checkbox chkbx_fractal "Fractal" checked:true tooltip:"Toggles the fractal noise mode for the chosen tracks on all selected objects."
			spinner spn_roughness "Roughness: " enabled:true scale:0.001 range:[0.0,1.0,0.0] align:#left
		)
		group "Interface settings:"
		(
			button btn_apply "Apply To Selected" width:140 height:16 tooltip:"Applies the interface settings for either track or both to the selected objects."
			button btn_defaults "Default Settings" width:140 height:16 tooltip:"Resets the interface to the default settings.\n\nNOTE: Does not affect the actual noise track values on the selected objects."
			button btn_getObjSettings "Get Object Settings" width:140 height:16 tooltip:"Populates the interface settings using a single selected object noise settings for either track."
		)
		group "Controller assignment:"
		(
			button btn_addNoise "Add Noise" width:140 height:16 align:#center tooltip:"Add a list controller then a noise controller to the second slot for the chosen track on all selected objects."
			button btn_removeNoise "Remove Noise" width:140 height:16 align:#center tooltip:"Converts the chosen noise tracks to Position XYZ or Euler XYZ controllers on all selected objects."
		)
		checkbutton chkbtn_selectionSets "Selection Sets UI >>>" width:140 height:22 align:#center tooltip:""
		group "Playback:"
		(
			button btn_rewind "<<" width:30 height:22 align:#left offset:[0,0] across:4 tooltip:"Rewind"
			checkbutton chkbtn_play " > " width:40 height:22 align:#center offset:[8,0] tooltip:"Play"
			label lbl_loop "Loop: " align:#right offset:[15,5]
			checkbox chkbx_loop "" checked:timeConfiguration.playbackLoop align:#right offset:[10,5] checked:true
		)
		button btn_about "About" align:#center
		
		on btn_about pressed do
		(
			try(destroyDialog g_LBTools_NM.aboutScript_rollout)catch()
			createDialog g_LBTools_NM.aboutScript_rollout
		)

		on noiseManager_Rollout moved pos do
		(
			g_LBTools_NM.dPos = pos
			if chkbtn_selectionSets.checked do
			(
				SetDialogPos g_LBTools_NM.noiseManagerSS_Rollout (pos + [175,0])
			)
		)
		
		on noiseManager_Rollout close do
		(
			try(destroyDialog g_LBTools_NM.noiseManagerSS_Rollout)catch()
			try(destroyDialog g_LBTools_NM.aboutScript_rollout)catch()
			stopAnimation()
		)
		
		on chkbtn_selectionSets changed theState do
		(
			if theState then
			(
				createDialog g_LBTools_NM.noiseManagerSS_Rollout width:360 pos:((GetDialogPos g_LBTools_NM.noiseManager_Rollout) + [175,0])
			)
			else
			(
				try(destroyDialog g_LBTools_NM.noiseManagerSS_Rollout)catch()
			)
		)
		
		on chkbx_pos changed theState do
		(
			if theState == false do
			(
				if chkbx_rot.checked == false do
				(
					chkbx_pos.checked = true
				)
			)
		)
		
		on chkbx_rot changed theState do
		(
			if theState == false do
			(
				if chkbx_pos.checked == false do
				(
					chkbx_rot.checked = true
				)
			)
		)
		
		mapped fn f_applyChange obj =
		(
			try
			(
				if chkbx_rot.checked do
				(
					local nt_index = g_LBTools_NM.f_findNoiseTrack_Rot obj
					if nt_index != undefined do
					(
						if nt_index == 0 then
						(
							obj.rotation.controller.seed = spn_seed.value
							obj.rotation.controller.frequency = spn_frequency.value
							obj.rotation.controller.noise_strength.x = (spn_strX.value * 0.0174533)
							obj.rotation.controller.noise_strength.y = (spn_strY.value * 0.0174533)
							obj.rotation.controller.noise_strength.z = (spn_strZ.value * 0.0174533)
							obj.rotation.controller.fractal = chkbx_fractal.checked
							obj.rotation.controller.roughness = spn_roughness.value
							obj.rotation.controller.x_positive = chkbx_posX.checked
							obj.rotation.controller.y_positive = chkbx_posY.checked
							obj.rotation.controller.z_positive = chkbx_posZ.checked
							obj.rotation.controller.rampIn = spn_rampIn.value
							obj.rotation.controller.rampOut = spn_rampOut.value
						)
						else
						(
							obj.rotation.controller[nt_index].seed = spn_seed.value
							obj.rotation.controller[nt_index].frequency = spn_frequency.value
							obj.rotation.controller[nt_index].noise_strength.x = (spn_strX.value * 0.0174533)
							obj.rotation.controller[nt_index].noise_strength.y = (spn_strY.value * 0.0174533)
							obj.rotation.controller[nt_index].noise_strength.z = (spn_strZ.value * 0.0174533)
							obj.rotation.controller[nt_index].fractal = chkbx_fractal.checked
							obj.rotation.controller[nt_index].roughness = spn_roughness.value
							obj.rotation.controller[nt_index].x_positive = chkbx_posX.checked
							obj.rotation.controller[nt_index].y_positive = chkbx_posY.checked
							obj.rotation.controller[nt_index].z_positive = chkbx_posZ.checked
							obj.rotation.controller[nt_index].rampIn = spn_rampIn.value
							obj.rotation.controller[nt_index].rampOut = spn_rampOut.value
						)
					)
				)		
				if chkbx_pos.checked do
				(
					local nt_index = g_LBTools_NM.f_findNoiseTrack_Pos obj
					if nt_index != undefined do
					(
						if nt_index == 0 then
						(
							obj.position.controller.seed = spn_seed.value
							obj.position.controller.frequency = spn_frequency.value
							obj.position.controller.noise_strength.x = spn_strX.value
							obj.position.controller.noise_strength.y = spn_strY.value
							obj.position.controller.noise_strength.z = spn_strZ.value
							obj.position.controller.fractal = chkbx_fractal.checked
							obj.position.controller.roughness = spn_roughness.value
							obj.position.controller.x_positive = chkbx_posX.checked
							obj.position.controller.y_positive = chkbx_posY.checked
							obj.position.controller.z_positive = chkbx_posZ.checked
							obj.position.controller.rampIn = spn_rampIn.value
							obj.position.controller.rampOut = spn_rampOut.value
						)
						else
						(
							obj.position.controller[nt_index].seed = spn_seed.value
							obj.position.controller[nt_index].frequency = spn_frequency.value
							obj.position.controller[nt_index].noise_strength.x = spn_strX.value
							obj.position.controller[nt_index].noise_strength.y = spn_strY.value
							obj.position.controller[nt_index].noise_strength.z = spn_strZ.value
							obj.position.controller[nt_index].fractal = chkbx_fractal.checked
							obj.position.controller[nt_index].roughness = spn_roughness.value
							obj.position.controller[nt_index].x_positive = chkbx_posX.checked
							obj.position.controller[nt_index].y_positive = chkbx_posY.checked
							obj.position.controller[nt_index].z_positive = chkbx_posZ.checked
							obj.position.controller[nt_index].rampIn = spn_rampIn.value
							obj.position.controller[nt_index].rampOut = spn_rampOut.value
						)
					)
				)
			)catch()
		)
		
		on btn_apply pressed do
		(
			if chkbtn_selectionSets.checked then
			(
				if g_LBTools_NM.a_SS_Items.count > 0 then
				(
					for obj in g_LBTools_NM.a_SS_Items do
					(
						f_applyChange g_LBTools_NM.a_SS_Items
					)
				)
				else
				(
					messageBox "Error: Select one or more objects in the selection set list with either Position OR Rotation noise list controllers assigned." title:""
				)
			)
			else
			(
				if selection.count > 0 then
				(
					f_applyChange (selection as array)
				)
				else
				(
					messageBox "Error: Select one or more objects with either Position OR Rotation noise list controllers assigned." title:""
				)
			)
		)
		
		fn f_getObjSettings obj =
		(
			if chkbx_rot.checked AND NOT chkbx_pos.checked do
			(
				try
				(
					local nt_index = g_LBTools_NM.f_findNoiseTrack_Rot obj
					if nt_index != undefined do
					(
						if nt_index == 0 then
						(
							spn_seed.value = obj.rotation.controller.seed
							spn_frequency.value = obj.rotation.controller.frequency
							spn_strX.value = (obj.rotation.controller.noise_strength.x / 0.0174533)
							spn_strY.value = (obj.rotation.controller.noise_strength.y / 0.0174533)
							spn_strZ.value = (obj.rotation.controller.noise_strength.z / 0.0174533)
							chkbx_fractal.checked = obj.rotation.controller.fractal
							spn_roughness.value = obj.rotation.controller.roughness
							chkbx_posX.checked = obj.rotation.controller.x_positive
							chkbx_posY.checked = obj.rotation.controller.y_positive
							chkbx_posZ.checked = obj.rotation.controller.z_positive
							spn_rampIn.value = obj.rotation.controller.rampIn
							spn_rampOut.value = obj.rotation.controller.rampOut
						)
						else
						(
							spn_seed.value = obj.rotation.controller[nt_index].seed
							spn_frequency.value = obj.rotation.controller[nt_index].frequency
							spn_strX.value = (obj.rotation.controller[nt_index].noise_strength.x / 0.0174533)
							spn_strY.value = (obj.rotation.controller[nt_index].noise_strength.y / 0.0174533)
							spn_strZ.value = (obj.rotation.controller[nt_index].noise_strength.z / 0.0174533)
							chkbx_fractal.checked = obj.rotation.controller[nt_index].fractal
							spn_roughness.value = obj.rotation.controller[nt_index].roughness
							chkbx_posX.checked = obj.rotation.controller[nt_index].x_positive
							chkbx_posY.checked = obj.rotation.controller[nt_index].y_positive
							chkbx_posZ.checked = obj.rotation.controller[nt_index].z_positive
							spn_rampIn.value = obj.rotation.controller[nt_index].rampIn
							spn_rampOut.value = obj.rotation.controller[nt_index].rampOut
						)
					)
				)catch()
			)
			if chkbx_pos.checked AND NOT chkbx_rot.checked do
			(
				try
				(
					local nt_index = g_LBTools_NM.f_findNoiseTrack_Pos obj
					if nt_index != undefined do
					(
						if nt_index == 0 then
						(
							spn_seed.value = obj.position.controller.seed
							spn_frequency.value = obj.position.controller.frequency
							spn_strX.value = obj.position.controller.noise_strength.x
							spn_strY.value = obj.position.controller.noise_strength.y
							spn_strZ.value = obj.position.controller.noise_strength.z
							chkbx_fractal.checked = obj.position.controller.fractal
							spn_roughness.value = obj.position.controller.roughness
							chkbx_posX.checked = obj.position.controller.x_positive
							chkbx_posY.checked = obj.position.controller.y_positive
							chkbx_posZ.checked = obj.position.controller.z_positive
							spn_rampIn.value = obj.position.controller.rampIn
							spn_rampOut.value = obj.position.controller.rampOut
						)
						else
						(
							spn_seed.value = obj.position.controller[nt_index].seed
							spn_frequency.value = obj.position.controller[nt_index].frequency
							spn_strX.value = obj.position.controller[nt_index].noise_strength.x
							spn_strY.value = obj.position.controller[nt_index].noise_strength.y
							spn_strZ.value = obj.position.controller[nt_index].noise_strength.z
							chkbx_fractal.checked = obj.position.controller[nt_index].fractal
							spn_roughness.value = obj.position.controller[nt_index].roughness
							chkbx_posX.checked = obj.position.controller[nt_index].x_positive
							chkbx_posY.checked = obj.position.controller[nt_index].y_positive
							chkbx_posZ.checked = obj.position.controller[nt_index].z_positive
							spn_rampIn.value = obj.position.controller[nt_index].rampIn
							spn_rampOut.value = obj.position.controller[nt_index].rampOut
						)
					)
				)catch()
			)
		)
		
		on btn_getObjSettings pressed do
		(
			if chkbx_rot.checked AND chkbx_pos.checked then
			(
				messageBox "Error: Settings can be retrieved from only ONE of the tracks - either Position OR Rotation." title:""
			)
			else
			(
				if chkbtn_selectionSets.checked then
				(
					if g_LBTools_NM.a_SS_Items.count == 1 then
					(
						f_getObjSettings g_LBTools_NM.a_SS_Items[1]
					)
					else
					(
						messageBox "Error: Select only ONE object from the selection set list with either Position OR Rotation noise list controllers assigned." title:""
					)
				)
				else
				(
					if selection.count == 1 then
					(
						f_getObjSettings selection[1]
					)
					else
					(
						messageBox "Error: Select only ONE object with either Position OR Rotation noise list controllers assigned." title:""
					)
				)
				if chkbx_fractal.checked then
				(
					spn_roughness.enabled = true
				)
				else
				(
					spn_roughness.enabled = false
				)
			)
		)
		
		on btn_defaults pressed do
		(
			chkbx_pos.checked = false
			chkbx_rot.checked = true
			spn_seed.value = 0
			spn_frequency.value = 0.5
			spn_strX.value = 45.0
			spn_strY.value = 45.0
			spn_strZ.value = 45.0
			chkbx_fractal.checked = true
			spn_roughness.enabled = true
			spn_roughness.value = 0.0
			chkbx_posX.checked = false
			chkbx_posY.checked = false
			chkbx_posZ.checked = false
			spn_rampIn.value = 0
			spn_rampOut.value = 0
		)
		
		mapped fn f_seedChange obj val =
		(
			try
			(
				if chkbx_rot.checked do
				(
					local nt_index = g_LBTools_NM.f_findNoiseTrack_Rot obj
					if nt_index != undefined do
					(
						if nt_index == 0 then
						(
							obj.rotation.controller.seed = val
						)
						else
						(
							obj.rotation.controller[nt_index].seed = val
						)
					)
				)
				if chkbx_pos.checked do
				(
					local nt_index = g_LBTools_NM.f_findNoiseTrack_Pos obj
					if nt_index != undefined do
					(
						if nt_index == 0 then
						(
							obj.position.controller.seed = val
						)
						else
						(
							obj.position.controller[nt_index].seed = val
						)
					)
				)
			)catch()
		)
		
		on spn_seed changed val do
		(
			if chkbtn_selectionSets.checked then
			(
				if g_LBTools_NM.a_SS_Items.count > 0 do
				(
					for obj in g_LBTools_NM.a_SS_Items do
					(
						f_seedChange g_LBTools_NM.a_SS_Items val
					)
				)
			)
			else
			(
				if selection.count > 0 do
				(
					f_seedChange (selection as array) val
				)
			)
		)
		
		mapped fn f_frequencyChange obj val =
		(
			try
			(
				if chkbx_rot.checked do
				(
					local nt_index = g_LBTools_NM.f_findNoiseTrack_Rot obj
					if nt_index != undefined do
					(
						if nt_index == 0 then
						(
							obj.rotation.controller.frequency = val
						)
						else
						(
							obj.rotation.controller[nt_index].frequency = val
						)
					)
				)
				if chkbx_pos.checked do
				(
					local nt_index = g_LBTools_NM.f_findNoiseTrack_Pos obj
					if nt_index != undefined do
					(
						if nt_index == 0 then
						(
							obj.position.controller.frequency = val
						)
						else
						(
							obj.position.controller[nt_index].frequency = val
						)
					)
				)
			)catch()
		)
		
		on spn_frequency changed val do
		(
			if chkbtn_selectionSets.checked then
			(
				if g_LBTools_NM.a_SS_Items.count > 0 do
				(
					f_frequencyChange g_LBTools_NM.a_SS_Items val
				)
			)
			else
			(
				if selection.count > 0 do
				(
					f_frequencyChange (selection as array) val
				)
			)
		)
		
		mapped fn f_strengthChange obj val axis =
		(
			try
			(
				if chkbx_rot.checked do
				(
					if chkbtn_lock.checked then
					(
						local nt_index = g_LBTools_NM.f_findNoiseTrack_Rot obj
						if nt_index != undefined do
						(
							if nt_index == 0 then
							(
								obj.rotation.controller.noise_strength.x = (val * 0.0174533)
								obj.rotation.controller.noise_strength.y = (val * 0.0174533)
								obj.rotation.controller.noise_strength.z = (val * 0.0174533)
							)
							else
							(
								obj.rotation.controller[nt_index].noise_strength.x = (val * 0.0174533)
								obj.rotation.controller[nt_index].noise_strength.y = (val * 0.0174533)
								obj.rotation.controller[nt_index].noise_strength.z = (val * 0.0174533)
							)
						)
					)
					else
					(
						local nt_index = g_LBTools_NM.f_findNoiseTrack_Rot obj
						if nt_index != undefined do
						(
							if nt_index == 0 then
							(
								case axis of
								(
									"x":obj.rotation.controller.noise_strength.x = (val * 0.0174533)
									"y":obj.rotation.controller.noise_strength.y = (val * 0.0174533)
									"z":obj.rotation.controller.noise_strength.z = (val * 0.0174533)
								)
							)
							else
							(
								case axis of
								(
									"x":obj.rotation.controller[nt_index].noise_strength.x = (val * 0.0174533)
									"y":obj.rotation.controller[nt_index].noise_strength.y = (val * 0.0174533)
									"z":obj.rotation.controller[nt_index].noise_strength.z = (val * 0.0174533)
								)
							)
						)
					)
				)
				if chkbx_pos.checked do
				(
					if chkbtn_lock.checked then
					(
						local nt_index = g_LBTools_NM.f_findNoiseTrack_Pos obj
						if nt_index != undefined do
						(
							if nt_index == 0 then
							(
								obj.position.controller.noise_strength.x = val
								obj.position.controller.noise_strength.y = val
								obj.position.controller.noise_strength.z = val
							)
							else
							(
								obj.position.controller[nt_index].noise_strength.x = val
								obj.position.controller[nt_index].noise_strength.y = val
								obj.position.controller[nt_index].noise_strength.z = val
							)
						)
					)
					else
					(
						local nt_index = g_LBTools_NM.f_findNoiseTrack_Pos obj
						if nt_index != undefined do
						(
							if nt_index == 0 then
							(
								case axis of
								(
									"x":obj.position.controller.noise_strength.x = val
									"y":obj.position.controller.noise_strength.y = val
									"z":obj.position.controller.noise_strength.z = val
								)
							)
							else
							(
								case axis of
								(
									"x":obj.position.controller[nt_index].noise_strength.x = val
									"y":obj.position.controller[nt_index].noise_strength.y = val
									"z":obj.position.controller[nt_index].noise_strength.z = val
								)
							)
						)
					)
				)
			)catch()
		)
		
		on spn_strX changed val do
		(
			if chkbtn_lock.checked do
			(
				spn_strY.value = spn_strX.value
				spn_strZ.value = spn_strX.value
			)
			if chkbtn_selectionSets.checked then
			(
				if g_LBTools_NM.a_SS_Items.count > 0 do
				(
					f_strengthChange g_LBTools_NM.a_SS_Items val "x"
				)
			)
			else
			(
				if selection.count > 0 do
				(
					f_strengthChange (selection as array) val "x"
				)
			)
		)

		on spn_strY changed val do
		(
			if chkbtn_lock.checked do
			(
				spn_strX.value = spn_strY.value
				spn_strZ.value = spn_strY.value
			)
			if chkbtn_selectionSets.checked then
			(
				if g_LBTools_NM.a_SS_Items.count > 0 do
				(
					f_strengthChange g_LBTools_NM.a_SS_Items val "y"
				)
			)
			else
			(
				if selection.count > 0 do
				(
					f_strengthChange (selection as array) val "y"
				)
			)
		)
		
		on spn_strZ changed val do
		(
			if chkbtn_lock.checked do
			(
				spn_strX.value = spn_strZ.value
				spn_strY.value = spn_strZ.value
			)
			if chkbtn_selectionSets.checked then
			(
				if g_LBTools_NM.a_SS_Items.count > 0 do
				(
					f_strengthChange g_LBTools_NM.a_SS_Items val "z"
				)
			)
			else
			(
				if selection.count > 0 do
				(
					f_strengthChange (selection as array) val "z"
				)
			)
		)

		mapped fn f_fractalChange obj theState =
		(
			try
			(
				if theState then
				(
					if chkbx_rot.checked do
					(
						local nt_index = g_LBTools_NM.f_findNoiseTrack_Rot obj
						if nt_index != undefined do
						(
							if nt_index == 0 then
							(
								obj.rotation.controller.fractal = true
							)
							else
							(
								obj.rotation.controller[nt_index].fractal = true
							)
						)
					)
					if chkbx_pos.checked do
					(
						local nt_index = g_LBTools_NM.f_findNoiseTrack_Pos obj
						if nt_index != undefined do
						(
							if nt_index == 0 then
							(
								obj.position.controller.fractal = true
							)
							else
							(
								obj.position.controller[nt_index].fractal = true
							)
						)
					)
				)
				else
				(
					if chkbx_rot.checked do
					(
						local nt_index = g_LBTools_NM.f_findNoiseTrack_Rot obj
						if nt_index != undefined do
						(
							if nt_index == 0 then
							(
								obj.rotation.controller.fractal = false
							)
							else
							(
								obj.rotation.controller[nt_index].fractal = false
							)
						)
					)
					if chkbx_pos.checked do
					(
						local nt_index = g_LBTools_NM.f_findNoiseTrack_Pos obj
						if nt_index != undefined do
						(
							if nt_index == 0 then
							(
								obj.position.controller.fractal = false
							)
							else
							(
								obj.position.controller[nt_index].fractal = false
							)
						)
					)
				)
			)catch()
		)
		
		on chkbx_fractal changed theState do
		(
			if theState then
			(
				spn_roughness.enabled = true
			)
			else
			(
				spn_roughness.enabled = false
			)
			if chkbtn_selectionSets.checked then
			(
				if g_LBTools_NM.a_SS_Items.count > 0 do
				(
					f_fractalChange g_LBTools_NM.a_SS_Items theState
				)
			)
			else
			(
				if selection.count > 0 do
				(
					f_fractalChange (selection as array) theState
				)
			)
		)
		
		mapped fn f_roughnessChange obj val =
		(
			try
			(
				if chkbx_rot.checked do
				(
					local nt_index = g_LBTools_NM.f_findNoiseTrack_Rot obj
					if nt_index != undefined do
					(
						if nt_index == 0 then
						(
							obj.rotation.controller.roughness = val
						)
						else
						(
							obj.rotation.controller[nt_index].roughness = val
						)
					)
				)
				if chkbx_pos.checked do
				(
					local nt_index = g_LBTools_NM.f_findNoiseTrack_Pos obj
					if nt_index != undefined do
					(
						if nt_index == 0 then
						(
							obj.position.controller.roughness = val
						)
						else
						(
							obj.position.controller[nt_index].roughness = val
						)
					)
				)
			)catch()
		)
		
		on spn_roughness changed val do
		(
			if chkbtn_selectionSets.checked then
			(
				if g_LBTools_NM.a_SS_Items.count > 0 do
				(
					f_roughnessChange g_LBTools_NM.a_SS_Items val
				)
			)
			else
			(
				if selection.count > 0 do
				(
					f_roughnessChange (selection as array) val
				)
			)
		)
		
		mapped fn f_positiveChange obj theState axis =
		(
			try
			(
				if theState then
				(
					if chkbx_rot.checked do
					(
						local nt_index = g_LBTools_NM.f_findNoiseTrack_Rot obj
						if nt_index != undefined do
						(
							if nt_index == 0 then
							(
								case axis of
								(
									"x":obj.rotation.controller.x_positive = true
									"y":obj.rotation.controller.y_positive = true
									"z":obj.rotation.controller.z_positive = true
								)
							)
							else
							(
								case axis of
								(
									"x":obj.rotation.controller[nt_index].x_positive = true
									"y":obj.rotation.controller[nt_index].y_positive = true
									"z":obj.rotation.controller[nt_index].z_positive = true
								)
							)
						)
					)
					if chkbx_pos.checked do
					(
						local nt_index = g_LBTools_NM.f_findNoiseTrack_Pos obj
						if nt_index != undefined do
						(
							if nt_index == 0 then
							(
								case axis of
								(
									"x":obj.position.controller.x_positive = true
									"y":obj.position.controller.y_positive = true
									"z":obj.position.controller.z_positive = true
								)
							)
							else
							(
								case axis of
								(
									"x":obj.position.controller[nt_index].x_positive = true
									"y":obj.position.controller[nt_index].y_positive = true
									"z":obj.position.controller[nt_index].z_positive = true
								)
							)
						)
					)
				)
				else
				(
					if chkbx_rot.checked do
					(
						local nt_index = g_LBTools_NM.f_findNoiseTrack_Rot obj
						if nt_index != undefined do
						(
							if nt_index == 0 then
							(
								case axis of
								(
									"x":obj.rotation.controller.x_positive = false
									"y":obj.rotation.controller.y_positive = false
									"z":obj.rotation.controller.z_positive = false
								)
							)
							else
							(
								case axis of
								(
									"x":obj.rotation.controller[nt_index].x_positive = false
									"y":obj.rotation.controller[nt_index].y_positive = false
									"z":obj.rotation.controller[nt_index].z_positive = false
								)
							)
						)
					)
					if chkbx_pos.checked do
					(
						local nt_index = g_LBTools_NM.f_findNoiseTrack_Pos obj
						if nt_index != undefined do
						(
							if nt_index == 0 then
							(
								case axis of
								(
									"x":obj.position.controller.x_positive = false
									"y":obj.position.controller.y_positive = false
									"z":obj.position.controller.z_positive = false
								)
							)
							else
							(
								case axis of
								(
									"x":obj.position.controller[nt_index].x_positive = false
									"y":obj.position.controller[nt_index].y_positive = false
									"z":obj.position.controller[nt_index].z_positive = false
								)
							)
						)
					)
				)
			)catch()
		)
		
		on chkbx_posX changed theState do
		(
			if chkbtn_selectionSets.checked then
			(
				if g_LBTools_NM.a_SS_Items.count > 0 do
				(
					f_positiveChange g_LBTools_NM.a_SS_Items theState "x"
				)
			)
			else
			(
				if selection.count > 0 do
				(
					f_positiveChange (selection as array) theState "x"
				)
			)
		)
		
		on chkbx_posY changed theState do
		(
			if chkbtn_selectionSets.checked then
			(
				if g_LBTools_NM.a_SS_Items.count > 0 do
				(
					f_positiveChange g_LBTools_NM.a_SS_Items theState "y"
				)
			)
			else
			(
				if selection.count > 0 do
				(
					f_positiveChange (selection as array) theState "y"
				)
			)
		)
		
		on chkbx_posZ changed theState do
		(
			if chkbtn_selectionSets.checked then
			(
				if g_LBTools_NM.a_SS_Items.count > 0 do
				(
					f_positiveChange g_LBTools_NM.a_SS_Items theState "z"
				)
			)
			else
			(
				if selection.count > 0 do
				(
					f_positiveChange (selection as array) theState "z"
				)
			)
		)
		
		mapped fn f_rampChange obj val type =
		(
			try
			(
				if chkbx_rot.checked do
				(
					local nt_index = g_LBTools_NM.f_findNoiseTrack_Rot obj
					if nt_index != undefined do
					(
						if nt_index == 0 then
						(
							case type of
							(
								"rampIn":obj.rotation.controller.rampIn = val
								"rampOut":obj.rotation.controller.rampOut = val
							)
						)
						else
						(
							case type of
							(
								"rampIn":obj.rotation.controller[nt_index].rampIn = val
								"rampOut":obj.rotation.controller[nt_index].rampOut = val
							)
						)
					)
				)
				if chkbx_pos.checked do
				(
					local nt_index = g_LBTools_NM.f_findNoiseTrack_Pos obj
					if nt_index != undefined do
					(
						if nt_index == 0 then
						(
							case type of
							(
								"rampIn":obj.position.controller.rampIn = val
								"rampOut":obj.position.controller.rampOut = val
							)
						)
						else
						(
							case type of
							(
								"rampIn":obj.position.controller[nt_index].rampIn = val
								"rampOut":obj.position.controller[nt_index].rampOut = val
							)
						)
					)
				)
			)catch()
		)
		
		on spn_rampIn changed val do
		(
			if chkbtn_selectionSets.checked then
			(
				if g_LBTools_NM.a_SS_Items.count > 0 do
				(
					f_rampChange g_LBTools_NM.a_SS_Items val "rampIn"
				)
			)
			else
			(
				if selection.count > 0 do
				(
					f_rampChange (selection as array) val "rampIn"
				)
			)
		)
		
		on spn_rampOut changed val do
		(
			if chkbtn_selectionSets.checked then
			(
				if g_LBTools_NM.a_SS_Items.count > 0 do
				(
					f_rampChange g_LBTools_NM.a_SS_Items val "rampOut"
				)
			)
			else
			(
				if selection.count > 0 do
				(
					f_rampChange (selection as array) val "rampOut"
				)
			)
		)
		
		mapped fn f_addNoise obj =
		(
			try
			(
				if chkbx_rot.checked do
				(
					obj.rotation.controller = Euler_XYZ() --// Remove existing list controllers or contraints
					obj.rotation.controller = Rotation_List()
					obj.rotation.controller.available.controller = Noise_Rotation()
				)
				if chkbx_pos.checked do
				(
					obj.position.controller = Position_XYZ() --// Remove existing list controllers or contraints
					obj.position.controller = Position_List()
					obj.position.controller.available.controller = Noise_Position()
				)
			)catch()
		)
		
		on btn_addNoise pressed do
		(
			if chkbtn_selectionSets.checked then
			(
				if g_LBTools_NM.a_SS_Items.count > 0 do
				(
					f_addNoise g_LBTools_NM.a_SS_Items
					g_LBTools_NM.f_refreshSS_UI()
				)
			)
			else
			(
				if selection.count > 0 do
				(
					f_addNoise (selection as array)
					if chkbx_rot.checked AND chkbx_pos.checked then
					(
						messageBox "Position and Rotation noise added to the selection." title:""
					)
					else
					(
						if chkbx_rot.checked do
						(
							messageBox "Rotation noise added to the selection." title:""
						)
						if chkbx_pos.checked do
						(
							messageBox "Position noise added to the selection." title:""
						)
					)
				)
			)
		)
		
		mapped fn f_removeNoise obj =
		(
			try
			(
				if chkbx_rot.checked do
				(
					obj.rotation.controller = Euler_XYZ()
				)
				if chkbx_pos.checked do
				(
					obj.position.controller = Position_XYZ()
				)
			)catch()
		)		
		
		on btn_removeNoise pressed do
		(
			if chkbtn_selectionSets.checked then
			(
				if g_LBTools_NM.a_SS_Items.count > 0 do
				(
					f_removeNoise g_LBTools_NM.a_SS_Items
					g_LBTools_NM.f_refreshSS_UI()
				)
			)
			else
			(
				if selection.count > 0 do
				(
					f_removeNoise (selection as array)
					if chkbx_rot.checked AND chkbx_pos.checked then
					(
						messageBox "Position and Rotation noise removed from the selection." title:""
					)
					else
					(
						if chkbx_rot.checked do
						(
							messageBox "Rotation noise removed from the selection." title:""
						)
						if chkbx_pos.checked do
						(
							messageBox "Position noise removed from the selection." title:""
						)
					)
				)
			)
		)
		
		on btn_rewind pressed do
		(
			stopAnimation()
			chkbtn_play.checked = false
			sliderTime = 0
		)
		
		on chkbtn_play changed theState do
		(
			if theState then
			(
				playAnimation()
				if sliderTime == animationRange.end do
				(
					chkbtn_play.checked = false
				)
			)
			else
			(
				stopAnimation()
			)
		)
		
		on chkbx_loop changed theState do
		(
			if timeConfiguration.playbackLoop then
			(
				timeConfiguration.playbackLoop = false
			)
			else
			(
				timeConfiguration.playbackLoop = true
			)
		)
	) --// end rollout
	
	rollout aboutScript_rollout "About"
	(
		label lbl_author "Author - Lorne Brooks"
		hyperLink myEmail "lbtools2k@gmail.com" color:(color 0 0 255) hovercolor:(color 0 0 190) visitedcolor:(color 190 0 0) align:#center address:"mailto:lbtools2k@gmail.com"
		label lbl_version "Version 1.2 - 27 Oct 2014"
	) --// end rollout
	
	--// Load UI
	try(destroyDialog noiseManager_Rollout)catch()
	if dPos == undefined do
	(
		dPos = mouse.screenpos + [-75,-15]
	)
	createDialog noiseManager_Rollout width:160 pos:dPos
)

) --// end struct

g_LBTools_NM = s_LBTools_noiseManager()

/*********************************************************************************************************************************************************************/

macroscript noiseManager
	category:"LB Tools"
	buttontext:"{ N-MAN }"
	tooltip:"Noise Manager 1.2"
(
	g_LBTools_NM.f_noiseManager()
)

/*********************************************************************************************************************************************************************/